import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import javax.jms.*;

import com.ibm.mq.jms.*;
import com.ibm.msg.client.wmq.WMQConstants;

/**
 * Program Name
 *  MQTestJMS12L
 *
 * Description
 *  This java JMS class will connect to a remote queue manager, loop getting messages until the queue is empty.
 *
 * Sample Command Line Parameters
 * bindings mode
 *  -m MQA1 -q TEST.Q1
 * client mode
 *  -m MQA1 -q TEST.Q1 -h 127.0.0.1 -p 1414 -c TEST.CHL -u UserID -x Password
 *
 * @author Roger Lacroix
 */
public class MQTestJMS12L
{
   private static final SimpleDateFormat  LOGGER_TIMESTAMP = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.SSS");

   private Hashtable<String,String> params;
   private MQQueueConnectionFactory mqQCF = null;


   /**
    * The constructor
    */
   public MQTestJMS12L()
   {
      super();
      params = new Hashtable<String,String>();
   }

   /**
    * Make sure the required parameters are present.
    * @return true/false
    */
   private boolean allParamsPresent()
   {
      boolean b = params.containsKey("-m") && params.containsKey("-q");

      if (params.containsKey("-c"))
      {
         b = b && params.containsKey("-c") && params.containsKey("-h") && params.containsKey("-p");
      }

      if (b)
      {
         try
         {
            if (params.containsKey("-p"))
               Integer.parseInt((String) params.get("-p"));
         }
         catch (NumberFormatException e)
         {
            b = false;
         }
      }

      return b;
   }

   /**
    * Extract the command-line parameters and initialize the MQ variables.
    * @param args
    * @throws IllegalArgumentException
    */
   private void init(String[] args) throws IllegalArgumentException
   {
      if (args.length > 0 && (args.length % 2) == 0)
      {
         for (int i = 0; i < args.length; i += 2)
         {
            params.put(args[i], args[i + 1]);
         }
      }
      else
      {
         throw new IllegalArgumentException();
      }

      if (allParamsPresent())
      {
         try
         {
            mqQCF = new MQQueueConnectionFactory();
            mqQCF.setQueueManager((String) params.get("-m"));
            
            if (params.containsKey("-c"))
            {
               mqQCF.setTransportType(WMQConstants.WMQ_CM_CLIENT);
               mqQCF.setHostName((String) params.get("-h"));
               mqQCF.setChannel((String) params.get("-c"));
               try
               {
                  mqQCF.setPort(Integer.parseInt((String) params.get("-p")));
               }
               catch (NumberFormatException e)
               {
                  mqQCF.setPort(1414);
               }
            }
            else
            {
               mqQCF.setTransportType(WMQConstants.WMQ_CM_BINDINGS);
            }
         }
         catch (JMSException e)
         {
            MQTestJMS12L.logger("getLinkedException()=" + e.getLinkedException());
            MQTestJMS12L.logger(e.getLocalizedMessage());
            e.printStackTrace();
            throw new IllegalArgumentException();
         }
         catch (Exception e)
         {
            MQTestJMS12L.logger(e.getLocalizedMessage());
            e.printStackTrace();
            throw new IllegalArgumentException();
         }
      }
      else
      {
         throw new IllegalArgumentException();
      }
   }

   /**
    * Test the connection to the queue manager.
    * @throws MQException
    */
   private void testConn()
   {
      QueueConnection conn = null;
      QueueSession session = null;
      Queue myQ = null;

      try
      {
         if (params.containsKey("-u"))
            conn = mqQCF.createQueueConnection((String) params.get("-u"), (String) params.get("-x"));
         else
            conn = mqQCF.createQueueConnection();
         conn.start();

         session = conn.createQueueSession(false, javax.jms.Session.AUTO_ACKNOWLEDGE);
         MQTestJMS12L.logger("successfully connected.");

         myQ = session.createQueue((String) params.get("-q"));
         MQTestJMS12L.logger("successfully opened "+ (String) params.get("-q"));

         MQDestination mqd = (MQDestination) myQ;
         mqd.setTargetClient(WMQConstants.WMQ_CLIENT_JMS_COMPLIANT);
//         mqd.setTargetClient(WMQConstants.WMQ_CLIENT_NONJMS_MQ);

         receiveMsg(session, myQ);
      }
      catch (JMSException e)
      {
         MQTestJMS12L.logger("getLinkedException()=" + e.getLinkedException());
         MQTestJMS12L.logger(e.getLocalizedMessage());
         e.printStackTrace();
      }
      catch (Exception e)
      {
         MQTestJMS12L.logger(e.getLocalizedMessage());
         e.printStackTrace();
      }
      finally
      {
         try
         {
            if (session != null)
            {
               session.close();
               MQTestJMS12L.logger("Closed session");
            }
         }
         catch (Exception ex)
         {
            MQTestJMS12L.logger("session.close() : " + ex.getLocalizedMessage());
         }

         try
         {
            if (conn != null)
            {
               conn.stop();
               MQTestJMS12L.logger("Stopped connection");
            }
         }
         catch (Exception ex)
         {
            MQTestJMS12L.logger("connection.stop() : " + ex.getLocalizedMessage());
         }

         try
         {
            if (conn != null)
            {
               conn.close();
               MQTestJMS12L.logger("Closed connection");
            }
         }
         catch (Exception ex)
         {
            MQTestJMS12L.logger("connection.close() : " + ex.getLocalizedMessage());
         }
      }
   }

   /**
    * Get a message from a queue.
    * @throws JMSException
    */
   private void receiveMsg(QueueSession session, Queue myQ) throws JMSException
   {
      boolean more = true;
      QueueReceiver receiver = null;
      Message msg = null;

      try
      {
         receiver = session.createReceiver(myQ);
         while (more)
         {
            msg = receiver.receiveNoWait();
            if (msg != null)
               MQTestJMS12L.logger("Incoming Msg>>>"+msg);
            else
               more = false;
         }
      }
      finally
      {
         try
         {
            if (receiver != null)
               receiver.close();
         }
         catch (Exception ex)
         {
            MQTestJMS12L.logger("sender.close() : " + ex.getLocalizedMessage());
         }
      }
   }

   /**
    * A simple logger method
    * @param data
    */
   public static void logger(String data)
   {
      String className = Thread.currentThread().getStackTrace()[2].getClassName();

      // Remove the package info.
      if ( (className != null) && (className.lastIndexOf('.') != -1) )
         className = className.substring(className.lastIndexOf('.')+1);

      System.out.println(LOGGER_TIMESTAMP.format(new Date())+" "+className+": "+Thread.currentThread().getStackTrace()[2].getMethodName()+": "+data);
   }

   /**
    * mainline
    * @param args
    */
   public static void main(String[] args)
   {
      MQTestJMS12L write = new MQTestJMS12L();

      try
      {
         write.init(args);
         write.testConn();
      }
      catch (IllegalArgumentException e)
      {
         System.err.println("Usage: java MQTestJMS12L -m QueueManagerName -q JMS_Queue_Name [-h host -p port -c channel] [-u UserID -x Password]");
         System.exit(1);
      }
      catch (Exception e)
      {
         System.err.println(e.getLocalizedMessage());
         System.exit(1);
      }

      System.exit(0);
   }
}